@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * Copyright (C) 2002 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2002-2004 Bastien Nocera <hadess@hadess.net>
  *
  * bacon-cd-selection.c
  *
@@ -125,38 +125,6 @@ bacon_cd_selection_class_init (BaconCdSelectionClass *klass)
 				G_TYPE_NONE, 1, G_TYPE_STRING);
 }
 
-static void
-bacon_cd_selection_init (BaconCdSelection *bcs)
-{
-	bcs->priv = g_new0 (BaconCdSelectionPrivate, 1);
-}
-
-static void
-bacon_cd_selection_finalize (GObject *object)
-{
-	GList *l;
-	BaconCdSelection *bcs = (BaconCdSelection *) object;
-
-	g_return_if_fail (bcs != NULL);
-	g_return_if_fail (BACON_IS_CD_SELECTION (bcs));
-
-	l = bcs->priv->cdroms;
-	while (l != NULL)
-	{
-		CDDrive *cdrom = l->data;
-
-		l = g_list_remove (l, cdrom);
-		cd_drive_free (cdrom);
-	}
-
-	g_free (bcs->priv);
-	bcs->priv = NULL;
-
-	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
-	}
-}
-
 static void
 combo_device_changed (GtkComboBox *combo, gpointer user_data)
 {
@@ -165,6 +133,13 @@ combo_device_changed (GtkComboBox *combo, gpointer user_data)
 	int i;
 
 	i = gtk_combo_box_get_active (combo);
+	/* No selection */
+	if (i < 0) {
+		g_signal_emit (G_OBJECT (bcs),
+				bcs_table_signals[DEVICE_CHANGED],
+				0, NULL);
+		return;
+	}
 	drive = get_drive (bcs, i);
 
 	g_signal_emit (G_OBJECT (bcs),
@@ -199,34 +174,60 @@ cdrom_combo_box (BaconCdSelection *bcs, gboolean show_recorders_only, gboolean s
 	}
 }
 
-GtkWidget *
-bacon_cd_selection_new (void)
+static void
+bacon_cd_selection_init (BaconCdSelection *bcs)
 {
-	GtkWidget *widget;
-	BaconCdSelection *bcs;
+	bcs->priv = g_new0 (BaconCdSelectionPrivate, 1);
+  
 	GtkCellRenderer *cell;
 	GtkListStore *store;
 
-	widget = GTK_WIDGET
-		(g_object_new (bacon_cd_selection_get_type (), NULL));
-
-	store = gtk_list_store_new (1, G_TYPE_STRING);
-	gtk_combo_box_set_model (GTK_COMBO_BOX (widget),
+  store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (bcs),
 			GTK_TREE_MODEL (store));
 
 	cell = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), cell, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), cell,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (bcs), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (bcs), cell,
 			"text", 0,
 			NULL);
 
-	bcs = BACON_CD_SELECTION (widget);
 	cdrom_combo_box (bcs, FALSE, FALSE);
 
 	g_signal_connect (G_OBJECT (bcs), "changed",
 			G_CALLBACK (combo_device_changed), bcs);
+}
 
-	return widget;
+static void
+bacon_cd_selection_finalize (GObject *object)
+{
+	GList *l;
+	BaconCdSelection *bcs = (BaconCdSelection *) object;
+
+	g_return_if_fail (bcs != NULL);
+	g_return_if_fail (BACON_IS_CD_SELECTION (bcs));
+
+	l = bcs->priv->cdroms;
+	while (l != NULL)
+	{
+		CDDrive *cdrom = l->data;
+
+		l = g_list_remove (l, cdrom);
+		cd_drive_free (cdrom);
+	}
+
+	g_free (bcs->priv);
+	bcs->priv = NULL;
+
+	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	}
+}
+
+GtkWidget *
+bacon_cd_selection_new (void)
+{
+  return GTK_WIDGET(g_object_new (bacon_cd_selection_get_type (), NULL));
 }
 
 static void
@@ -255,11 +256,21 @@ bacon_cd_selection_set_have_file_image (BaconCdSelection *bcs,
 		cd_drive_free (cdrom);
 		bcs->priv->cdroms = g_list_delete_link
 			(bcs->priv->cdroms, item);
+		gtk_widget_set_sensitive (GTK_WIDGET (bcs), (bcs->priv->cdroms != NULL));
 	} else {
+		gboolean activate = FALSE;
+
 		cdrom = cd_drive_get_file_image ();
 		gtk_combo_box_append_text (GTK_COMBO_BOX (bcs),
 				cdrom->display_name);
+		if (bcs->priv->cdroms == NULL) {
+			activate = TRUE;
+		}
 		bcs->priv->cdroms = g_list_append (bcs->priv->cdroms, cdrom);
+		gtk_widget_set_sensitive (GTK_WIDGET (bcs), TRUE);
+		if (activate != FALSE) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (bcs), 0);
+		}
 	}
 
 	bcs->priv->have_file_image = have_file_image;
@@ -314,11 +325,20 @@ bacon_cd_selection_set_recorders_only (BaconCdSelection *bcs,
 
 			l = prev;
 		}
+
+		/* Removing entries from the combo box may have invalidated
+		 * the currently selected one 
+		 */
+		if (gtk_combo_box_get_active (GTK_COMBO_BOX (bcs)) == -1) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (bcs), 0);
+		}
 	} else {
 		GList *drives = scan_for_cdroms (recorders_only, bcs->priv->have_file_image);
 		GList *l;
 		int i = g_list_length (bcs->priv->cdroms);
 
+		gtk_widget_set_sensitive (GTK_WIDGET (bcs), (drives != NULL));
+
 		if (bcs->priv->have_file_image)
 			i--;
 
